@@ -1,13 +1,11 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2017 The PHP Group                                |
+  | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -16,8 +14,6 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -33,7 +29,7 @@
 
 static inline ub4 pdo_oci_sanitize_prefetch(long prefetch);
 
-static int pdo_oci_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info) /* {{{ */
+static void pdo_oci_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 	pdo_oci_error_info *einfo;
@@ -52,8 +48,6 @@ static int pdo_oci_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info
 		add_next_index_long(info, einfo->errcode);
 		add_next_index_string(info, einfo->errmsg);
 	}
-
-	return 1;
 }
 /* }}} */
 
@@ -132,12 +126,6 @@ ub4 _oci_error(OCIError *err, pdo_dbh_t *dbh, pdo_stmt_t *stmt, char *what, swor
 					zend_bailout();
 					break;
 
-#if 0
-				case 955:	/* ORA-00955: name is already used by an existing object */
-					*pdo_err = PDO_ERR_ALREADY_EXISTS;
-					break;
-#endif
-
 				case 12154:	/* ORA-12154: TNS:could not resolve service name */
 					strcpy(*pdo_err, "42S02");
 					break;
@@ -187,7 +175,7 @@ ub4 _oci_error(OCIError *err, pdo_dbh_t *dbh, pdo_stmt_t *stmt, char *what, swor
 	}
 
 	/* little mini hack so that we can use this code from the dbh ctor */
-	if (!dbh->methods) {
+	if (!dbh->methods && status != OCI_SUCCESS_WITH_INFO) {
 		zend_throw_exception_ex(php_pdo_get_exception(), einfo->errcode, "SQLSTATE[%s]: %s", *pdo_err, einfo->errmsg);
 	}
 
@@ -195,7 +183,7 @@ ub4 _oci_error(OCIError *err, pdo_dbh_t *dbh, pdo_stmt_t *stmt, char *what, swor
 }
 /* }}} */
 
-static int oci_handle_closer(pdo_dbh_t *dbh) /* {{{ */
+static void oci_handle_closer(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 
@@ -243,21 +231,18 @@ static int oci_handle_closer(pdo_dbh_t *dbh) /* {{{ */
 	}
 
 	pefree(H, dbh->is_persistent);
-
-	return 0;
 }
 /* }}} */
 
-static int oci_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len, pdo_stmt_t *stmt, zval *driver_options) /* {{{ */
+static bool oci_handle_preparer(pdo_dbh_t *dbh, zend_string *sql, pdo_stmt_t *stmt, zval *driver_options) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 	pdo_oci_stmt *S = ecalloc(1, sizeof(*S));
 	ub4 prefetch;
-	char *nsql = NULL;
-	size_t nsql_len = 0;
+	zend_string *nsql = NULL;
 	int ret;
 
-#if HAVE_OCISTMTFETCH2
+#ifdef HAVE_OCISTMTFETCH2
 	S->exec_type = pdo_attr_lval(driver_options, PDO_ATTR_CURSOR,
 		PDO_CURSOR_FWDONLY) == PDO_CURSOR_SCROLL ?
 		OCI_STMT_SCROLLABLE_READONLY : OCI_DEFAULT;
@@ -267,17 +252,16 @@ static int oci_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len,
 
 	S->H = H;
 	stmt->supports_placeholders = PDO_PLACEHOLDER_NAMED;
-	ret = pdo_parse_params(stmt, (char*)sql, sql_len, &nsql, &nsql_len);
+	ret = pdo_parse_params(stmt, sql, &nsql);
 
 	if (ret == 1) {
 		/* query was re-written */
 		sql = nsql;
-		sql_len = nsql_len;
 	} else if (ret == -1) {
 		/* couldn't grok it */
 		strcpy(dbh->error_code, stmt->error_code);
 		efree(S);
-		return 0;
+		return false;
 	}
 
 	/* create an OCI statement handle */
@@ -286,10 +270,10 @@ static int oci_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len,
 	/* and our own private error handle */
 	OCIHandleAlloc(H->env, (dvoid*)&S->err, OCI_HTYPE_ERROR, 0, NULL);
 
-	if (sql_len) {
-		H->last_err = OCIStmtPrepare(S->stmt, H->err, (text*)sql, (ub4) sql_len, OCI_NTV_SYNTAX, OCI_DEFAULT);
+	if (ZSTR_LEN(sql) != 0) {
+		H->last_err = OCIStmtPrepare(S->stmt, H->err, (text*) ZSTR_VAL(sql), (ub4) ZSTR_LEN(sql), OCI_NTV_SYNTAX, OCI_DEFAULT);
 		if (nsql) {
-			efree(nsql);
+			zend_string_release(nsql);
 			nsql = NULL;
 		}
 		if (H->last_err) {
@@ -297,7 +281,7 @@ static int oci_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len,
 			OCIHandleFree(S->stmt, OCI_HTYPE_STMT);
 			OCIHandleFree(S->err, OCI_HTYPE_ERROR);
 			efree(S);
-			return 0;
+			return false;
 		}
 
 	}
@@ -314,15 +298,15 @@ static int oci_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len,
 	stmt->driver_data = S;
 	stmt->methods = &oci_stmt_methods;
 	if (nsql) {
-		efree(nsql);
+		zend_string_release(nsql);
 		nsql = NULL;
 	}
 
-	return 1;
+	return true;
 }
 /* }}} */
 
-static zend_long oci_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sql_len) /* {{{ */
+static zend_long oci_handle_doer(pdo_dbh_t *dbh, const zend_string *sql) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 	OCIStmt		*stmt;
@@ -332,7 +316,7 @@ static zend_long oci_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sql_len
 
 	OCIHandleAlloc(H->env, (dvoid*)&stmt, OCI_HTYPE_STMT, 0, NULL);
 
-	H->last_err = OCIStmtPrepare(stmt, H->err, (text*)sql, (ub4) sql_len, OCI_NTV_SYNTAX, OCI_DEFAULT);
+	H->last_err = OCIStmtPrepare(stmt, H->err, (text*)ZSTR_VAL(sql), (ub4) ZSTR_LEN(sql), OCI_NTV_SYNTAX, OCI_DEFAULT);
 	if (H->last_err) {
 		H->last_err = oci_drv_error("OCIStmtPrepare");
 		OCIHandleFree(stmt, OCI_HTYPE_STMT);
@@ -352,9 +336,13 @@ static zend_long oci_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sql_len
 	H->last_err = OCIStmtExecute(H->svc, stmt, H->err, 1, 0, NULL, NULL,
 			(dbh->auto_commit && !dbh->in_txn) ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT);
 
-	if (H->last_err) {
+	sword last_err = H->last_err;
+
+	if (last_err) {
 		H->last_err = oci_drv_error("OCIStmtExecute");
-	} else {
+	}
+
+	if (!last_err || last_err == OCI_SUCCESS_WITH_INFO) {
 		/* return the number of affected rows */
 		H->last_err = OCIAttrGet(stmt, OCI_HTYPE_STMT, &rowcount, 0, OCI_ATTR_ROW_COUNT, H->err);
 		ret = rowcount;
@@ -366,51 +354,52 @@ static zend_long oci_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sql_len
 }
 /* }}} */
 
-static int oci_handle_quoter(pdo_dbh_t *dbh, const char *unquoted, size_t unquotedlen, char **quoted, size_t *quotedlen, enum pdo_param_type paramtype ) /* {{{ */
+static zend_string* oci_handle_quoter(pdo_dbh_t *dbh, const zend_string *unquoted, enum pdo_param_type paramtype ) /* {{{ */
 {
 	int qcount = 0;
 	char const *cu, *l, *r;
-	char *c;
+	char *c, *quoted;
+	size_t quotedlen;
+	zend_string *quoted_str;
 
-	if (!unquotedlen) {
-		*quotedlen = 2;
-		*quoted = emalloc(*quotedlen+1);
-		strcpy(*quoted, "''");
-		return 1;
+	if (ZSTR_LEN(unquoted) == 0) {
+		return zend_string_init("''", 2, 0);
 	}
 
 	/* count single quotes */
-	for (cu = unquoted; (cu = strchr(cu,'\'')); qcount++, cu++)
+	for (cu = ZSTR_VAL(unquoted); (cu = strchr(cu,'\'')); qcount++, cu++)
 		; /* empty loop */
 
-	*quotedlen = unquotedlen + qcount + 2;
-	*quoted = c = emalloc(*quotedlen+1);
+	quotedlen = ZSTR_LEN(unquoted) + qcount + 2;
+	quoted = c = emalloc(quotedlen+1);
 	*c++ = '\'';
 
 	/* foreach (chunk that ends in a quote) */
-	for (l = unquoted; (r = strchr(l,'\'')); l = r+1) {
+	for (l = ZSTR_VAL(unquoted); (r = strchr(l,'\'')); l = r+1) {
 		strncpy(c, l, r-l+1);
 		c += (r-l+1);
 		*c++ = '\'';			/* add second quote */
 	}
 
-    /* Copy remainder and add enclosing quote */
-	strncpy(c, l, *quotedlen-(c-*quoted)-1);
-	(*quoted)[*quotedlen-1] = '\'';
-	(*quoted)[*quotedlen]   = '\0';
+	/* Copy remainder and add enclosing quote */
+	strncpy(c, l, quotedlen-(c-quoted)-1);
+	quoted[quotedlen-1] = '\'';
+	quoted[quotedlen]   = '\0';
 
-	return 1;
+	quoted_str = zend_string_init(quoted, quotedlen, 0);
+	efree(quoted);
+	return quoted_str;
 }
 /* }}} */
 
-static int oci_handle_begin(pdo_dbh_t *dbh) /* {{{ */
+static bool oci_handle_begin(pdo_dbh_t *dbh) /* {{{ */
 {
 	/* with Oracle, there is nothing special to be done */
-	return 1;
+	return true;
 }
 /* }}} */
 
-static int oci_handle_commit(pdo_dbh_t *dbh) /* {{{ */
+static bool oci_handle_commit(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 
@@ -418,13 +407,13 @@ static int oci_handle_commit(pdo_dbh_t *dbh) /* {{{ */
 
 	if (H->last_err) {
 		H->last_err = oci_drv_error("OCITransCommit");
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 /* }}} */
 
-static int oci_handle_rollback(pdo_dbh_t *dbh) /* {{{ */
+static bool oci_handle_rollback(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 
@@ -432,36 +421,155 @@ static int oci_handle_rollback(pdo_dbh_t *dbh) /* {{{ */
 
 	if (H->last_err) {
 		H->last_err = oci_drv_error("OCITransRollback");
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 /* }}} */
 
-static int oci_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *val) /* {{{ */
+static bool oci_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *val) /* {{{ */
 {
-	zend_long lval = zval_get_long(val);
+	zend_long lval;
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 
-	if (attr == PDO_ATTR_AUTOCOMMIT) {
-		if (dbh->in_txn) {
-			/* Assume they want to commit whatever is outstanding */
-			H->last_err = OCITransCommit(H->svc, H->err, 0);
-
-			if (H->last_err) {
-				H->last_err = oci_drv_error("OCITransCommit");
-				return 0;
+	switch (attr) {
+		case PDO_ATTR_AUTOCOMMIT:
+		{
+			bool bval;
+			if (!pdo_get_bool_param(&bval, val)) {
+				return false;
 			}
-			dbh->in_txn = 0;
-		}
 
-		dbh->auto_commit = (unsigned int)lval? 1 : 0;
-		return 1;
-	} else if (attr == PDO_ATTR_PREFETCH) {
-		H->prefetch = pdo_oci_sanitize_prefetch(lval);
-		return 1;
-	} else {
-		return 0;
+			if (dbh->in_txn) {
+				/* Assume they want to commit whatever is outstanding */
+				H->last_err = OCITransCommit(H->svc, H->err, 0);
+
+				if (H->last_err) {
+					H->last_err = oci_drv_error("OCITransCommit");
+					return false;
+				}
+				dbh->in_txn = false;
+			}
+
+			dbh->auto_commit = (unsigned int) bval;
+			return true;
+		}
+		case PDO_ATTR_PREFETCH:
+		{
+			if (!pdo_get_long_param(&lval, val)) {
+				return false;
+			}
+
+			H->prefetch = pdo_oci_sanitize_prefetch(lval);
+			return true;
+		}
+		case PDO_OCI_ATTR_ACTION:
+		{
+#if (OCI_MAJOR_VERSION >= 10)
+			zend_string *action = zval_try_get_string(val);
+			if (UNEXPECTED(!action)) {
+				return false;
+			}
+
+			H->last_err = OCIAttrSet(H->session, OCI_HTYPE_SESSION,
+				(dvoid *) ZSTR_VAL(action), (ub4) ZSTR_LEN(action),
+				OCI_ATTR_ACTION, H->err);
+			if (H->last_err) {
+				oci_drv_error("OCIAttrSet: OCI_ATTR_ACTION");
+				return false;
+			}
+			return true;
+#else
+			oci_drv_error("Unsupported attribute type");
+			return false;
+#endif
+		}
+		case PDO_OCI_ATTR_CLIENT_INFO:
+		{
+#if (OCI_MAJOR_VERSION >= 10)
+			zend_string *client_info = zval_try_get_string(val);
+			if (UNEXPECTED(!client_info)) {
+				return false;
+			}
+
+			H->last_err = OCIAttrSet(H->session, OCI_HTYPE_SESSION,
+				(dvoid *) ZSTR_VAL(client_info), (ub4) ZSTR_LEN(client_info),
+				OCI_ATTR_CLIENT_INFO, H->err);
+			if (H->last_err) {
+				oci_drv_error("OCIAttrSet: OCI_ATTR_CLIENT_INFO");
+				return false;
+			}
+			return true;
+#else
+			oci_drv_error("Unsupported attribute type");
+			return false;
+#endif
+		}
+		case PDO_OCI_ATTR_CLIENT_IDENTIFIER:
+		{
+#if (OCI_MAJOR_VERSION >= 10)
+			zend_string *identifier = zval_try_get_string(val);
+			if (UNEXPECTED(!identifier)) {
+				return false;
+			}
+
+			H->last_err = OCIAttrSet(H->session, OCI_HTYPE_SESSION,
+				(dvoid *) ZSTR_VAL(identifier), (ub4) ZSTR_LEN(identifier),
+				OCI_ATTR_CLIENT_IDENTIFIER, H->err);
+			if (H->last_err) {
+				oci_drv_error("OCIAttrSet: OCI_ATTR_CLIENT_IDENTIFIER");
+				return false;
+			}
+			return true;
+#else
+			oci_drv_error("Unsupported attribute type");
+			return false;
+#endif
+		}
+		case PDO_OCI_ATTR_MODULE:
+		{
+#if (OCI_MAJOR_VERSION >= 10)
+			zend_string *module = zval_try_get_string(val);
+			if (UNEXPECTED(!module)) {
+				return false;
+			}
+
+			H->last_err = OCIAttrSet(H->session, OCI_HTYPE_SESSION,
+				(dvoid *) ZSTR_VAL(module), (ub4) ZSTR_LEN(module),
+				OCI_ATTR_MODULE, H->err);
+			if (H->last_err) {
+				oci_drv_error("OCIAttrSet: OCI_ATTR_MODULE");
+				return false;
+			}
+			return true;
+#else
+			oci_drv_error("Unsupported attribute type");
+			return false;
+#endif
+		}
+		case PDO_OCI_ATTR_CALL_TIMEOUT:
+		{
+#if (OCI_MAJOR_VERSION >= 18)
+			if (!pdo_get_long_param(&lval, val)) {
+				return false;
+			}
+			ub4 timeout = (ub4) lval;
+
+			H->last_err = OCIAttrSet(H->svc, OCI_HTYPE_SVCCTX,
+				(dvoid *) &timeout, (ub4) 0,
+				OCI_ATTR_CALL_TIMEOUT, H->err);
+			if (H->last_err) {
+				oci_drv_error("OCIAttrSet: OCI_ATTR_CALL_TIMEOUT");
+				return false;
+			}
+			return true;
+#else
+			oci_drv_error("Unsupported attribute type");
+			return false;
+#endif
+		}
+		default:
+			return false;
 	}
 
 }
@@ -527,6 +635,26 @@ static int oci_handle_get_attribute(pdo_dbh_t *dbh, zend_long attr, zval *return
 		case PDO_ATTR_PREFETCH:
 			ZVAL_LONG(return_value, H->prefetch);
 			return TRUE;
+		case PDO_OCI_ATTR_CALL_TIMEOUT:
+		{
+#if (OCI_MAJOR_VERSION >= 18)
+			ub4 timeout;
+
+			H->last_err = OCIAttrGet(H->svc, OCI_HTYPE_SVCCTX,
+				(dvoid *) &timeout, NULL,
+				OCI_ATTR_CALL_TIMEOUT, H->err);
+			if (H->last_err) {
+				oci_drv_error("OCIAttrGet: OCI_ATTR_CALL_TIMEOUT");
+				return FALSE;
+			}
+
+			ZVAL_LONG(return_value, (zend_long) timeout);
+			return TRUE;
+#else
+			oci_drv_error("Unsupported attribute type");
+			return FALSE;
+#endif
+		}
 		default:
 			return FALSE;
 
@@ -536,7 +664,7 @@ static int oci_handle_get_attribute(pdo_dbh_t *dbh, zend_long attr, zval *return
 }
 /* }}} */
 
-static int pdo_oci_check_liveness(pdo_dbh_t *dbh) /* {{{ */
+static zend_result pdo_oci_check_liveness(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_oci_db_handle *H = (pdo_oci_db_handle *)dbh->driver_data;
 	sb4 error_code = 0;
@@ -575,7 +703,7 @@ static int pdo_oci_check_liveness(pdo_dbh_t *dbh) /* {{{ */
 }
 /* }}} */
 
-static struct pdo_dbh_methods oci_methods = {
+static const struct pdo_dbh_methods oci_methods = {
 	oci_handle_closer,
 	oci_handle_preparer,
 	oci_handle_doer,
@@ -584,11 +712,14 @@ static struct pdo_dbh_methods oci_methods = {
 	oci_handle_commit,
 	oci_handle_rollback,
 	oci_handle_set_attribute,
-	NULL,
+	NULL, /* last_id not supported */
 	pdo_oci_fetch_error_func,
 	oci_handle_get_attribute,
 	pdo_oci_check_liveness,	/* check_liveness */
-	NULL	/* get_driver_methods */
+	NULL, /* get_driver_methods */
+	NULL, /* request_shutdown */
+	NULL, /* in transaction, use PDO's internal tracking mechanism */
+	NULL /* get_gc */
 };
 
 static int pdo_oci_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /* {{{ */
@@ -597,18 +728,25 @@ static int pdo_oci_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /* {{{ *
 	int i, ret = 0;
 	struct pdo_data_src_parser vars[] = {
 		{ "charset",  NULL,	0 },
-		{ "dbname",   "",	0 }
+		{ "dbname",   "",	0 },
+		{ "user",     NULL, 0 },
+		{ "password", NULL, 0 }
 	};
 
-	php_pdo_parse_data_source(dbh->data_source, dbh->data_source_len, vars, 2);
+	php_pdo_parse_data_source(dbh->data_source, dbh->data_source_len, vars, 4);
 
 	H = pecalloc(1, sizeof(*H), dbh->is_persistent);
 	dbh->driver_data = H;
 
+	dbh->skip_param_evt =
+		1 << PDO_PARAM_EVT_FETCH_PRE |
+		1 << PDO_PARAM_EVT_FETCH_POST |
+		1 << PDO_PARAM_EVT_NORMALIZE;
+
 	H->prefetch = PDO_OCI_PREFETCH_DEFAULT;
 
 	/* allocate an environment */
-#if HAVE_OCIENVNLSCREATE
+#ifdef HAVE_OCIENVNLSCREATE
 	if (vars[0].optval) {
 		H->charset = OCINlsCharSetNameToId(pdo_oci_Env, (const oratext *)vars[0].optval);
 		if (!H->charset) {
@@ -664,6 +802,10 @@ static int pdo_oci_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /* {{{ *
 	}
 
 	/* username */
+	if (!dbh->username && vars[2].optval) {
+		dbh->username = pestrdup(vars[2].optval, dbh->is_persistent);
+	}
+
 	if (dbh->username) {
 		H->last_err = OCIAttrSet(H->session, OCI_HTYPE_SESSION,
 			   	dbh->username, (ub4) strlen(dbh->username),
@@ -675,6 +817,10 @@ static int pdo_oci_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /* {{{ *
 	}
 
 	/* password */
+	if (!dbh->password && vars[3].optval) {
+		dbh->password = pestrdup(vars[3].optval, dbh->is_persistent);
+	}
+
 	if (dbh->password) {
 		H->last_err = OCIAttrSet(H->session, OCI_HTYPE_SESSION,
 			   	dbh->password, (ub4) strlen(dbh->password),
@@ -689,7 +835,12 @@ static int pdo_oci_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /* {{{ *
 	H->last_err = OCISessionBegin(H->svc, H->err, H->session, OCI_CRED_RDBMS, OCI_DEFAULT);
 	if (H->last_err) {
 		oci_drv_error("OCISessionBegin");
-		goto cleanup;
+		/* OCISessionBegin returns OCI_SUCCESS_WITH_INFO when
+		 * user's password has expired, but is still usable.
+		 */
+		if (H->last_err != OCI_SUCCESS_WITH_INFO) {
+			goto cleanup;
+		}
 	}
 
 	/* set the server handle into service handle */
@@ -699,6 +850,13 @@ static int pdo_oci_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /* {{{ *
 		goto cleanup;
 	}
 
+	/* Get max character width */
+	H->last_err = OCINlsNumericInfoGet(H->env, H->err, &H->max_char_width, OCI_NLS_CHARSET_MAXBYTESZ);
+	if (H->last_err) {
+		oci_drv_error("OCINlsNumericInfoGet: OCI_NLS_CHARSET_MAXBYTESZ");
+		goto cleanup;
+	}
+
 	dbh->methods = &oci_methods;
 	dbh->alloc_own_columns = 1;
 	dbh->native_case = PDO_CASE_UPPER;
@@ -720,7 +878,7 @@ cleanup:
 }
 /* }}} */
 
-pdo_driver_t pdo_oci_driver = {
+const pdo_driver_t pdo_oci_driver = {
 	PDO_DRIVER_HEADER(oci),
 	pdo_oci_handle_factory
 };
@@ -735,13 +893,3 @@ static inline ub4 pdo_oci_sanitize_prefetch(long prefetch) /* {{{ */
 	return ((ub4)prefetch);
 }
 /* }}} */
-
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
